#[derive(EventMetadata)]
{
// Attributes available to this derive:
#[event_version]
}
NOTE: This is an internal implementation for #[near_bindgen(events(standard = ...))] attribute.
Please use near instead.
This derive macro is used to inject the necessary wrapper and logic to auto format
standard event logs and generate the emit function, and event version.
The macro is not for public use.